@@ -24,15 +24,15 @@
  */
 
 
-#define _GNU_SOURCE
 #include <stdio.h>
 #include <time.h>
-#undef _GNU_SOURCE // avoid multiple define from Python.h
 
 #include <Python.h>
 #include <datetime.h>
 
-static PyObject* CBSONError;
+#include "time_helpers.h"
+#include "encoding_helpers.h"
+
 static PyObject* InvalidName;
 static PyObject* InvalidDocument;
 static PyObject* InvalidStringData;
@@ -52,182 +52,41 @@ typedef int Py_ssize_t;
 
 #define INITIAL_BUFFER_SIZE 256
 
+/* Maximum number of regex flags */
+#define FLAGS_SIZE 7
 
-/* TODO we ought to check that the malloc or asprintf was successful
- * and raise an exception if not. */
 #if defined(WIN32) || defined(_MSC_VER)
-
-/* no mkgmtime on MSVC before VS 2005. this is terribly gross */
+/* This macro is basically an implementation of asprintf for win32
+ * We get the length of the int as string and malloc a buffer for it,
+ * returning -1 if that malloc fails. We then actually print to the
+ * buffer to get the string value as an int. Like asprintf, the result
+ * must be explicitly free'd when done being used.
+ */
 #if defined(_MSC_VER) && (_MSC_VER >= 1400)
-#define GMTIME_INVERSE(time_struct) _mkgmtime64(time_struct)
+#define INT2STRING(buffer, i)                                           \
+    *(buffer) = malloc(_scprintf("%d", (i)) + 1),                       \
+        (!(buffer) ?                                                    \
+         -1 :                                                           \
+         _snprintf_s(*(buffer),                                         \
+                     _scprintf("%d", (i)) + 1,                          \
+                     _scprintf("%d", (i)) + 1,                          \
+                     "%d",                                              \
+                     (i)))
+#define STRCAT(dest, n, src) strcat_s((dest), (n), (src))
 #else
-/* Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The name "Carnegie Mellon University" must not be used to
- *    endorse or promote products derived from this software without
- *    prior written permission. For permission or any other legal
- *    details, please contact
- *      Office of Technology Transfer
- *      Carnegie Mellon University
- *      5000 Forbes Avenue
- *      Pittsburgh, PA  15213-3890
- *      (412) 268-4387, fax: (412) 268-7395
- *      tech-transfer@andrew.cmu.edu
- *
- * 4. Redistributions of any form whatsoever must retain the following
- *    acknowledgment:
- *    "This product includes software developed by Computing Services
- *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
- *
- * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
- * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
- * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
- * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
- * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- *
- */
-/*
- * Copyright (c) 1987, 1989, 1993
- *    The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Arthur David Olson of the National Cancer Institute.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by the University of
- *    California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-/*
-** Adapted from code provided by Robert Elz, who writes:
-**    The "best" way to do mktime I think is based on an idea of Bob
-**    Kridle's (so its said...) from a long time ago. (mtxinu!kridle now).
-**    It does a binary search of the time_t space.  Since time_t's are
-**    just 32 bits, its a max of 32 iterations (even at 64 bits it
-**    would still be very reasonable).
-*/
-#ifndef WRONG
-#define WRONG     (-1)
-#endif /* !defined WRONG */
-
-static int tmcomp(atmp, btmp)
-     register const struct tm * const atmp;
-     register const struct tm * const btmp;
-{
-    register int      result;
-
-    if ((result = (atmp->tm_year - btmp->tm_year)) == 0 &&
-        (result = (atmp->tm_mon - btmp->tm_mon)) == 0 &&
-        (result = (atmp->tm_mday - btmp->tm_mday)) == 0 &&
-        (result = (atmp->tm_hour - btmp->tm_hour)) == 0 &&
-        (result = (atmp->tm_min - btmp->tm_min)) == 0)
-        result = atmp->tm_sec - btmp->tm_sec;
-    return result;
-}
-
-time_t mkgmtime(tmp)
-struct tm * const tmp;
-{
-    register int                  dir;
-    register int                  bits;
-    register int                  saved_seconds;
-    time_t                        t;
-    struct tm               yourtm, *mytm;
-
-    yourtm = *tmp;
-    saved_seconds = yourtm.tm_sec;
-    yourtm.tm_sec = 0;
-
-    /*
-    ** Calculate the number of magnitude bits in a time_t
-    ** (this works regardless of whether time_t is
-    ** signed or unsigned, though lint complains if unsigned).
-    */
-    for (bits = 0, t = 1; t > 0; ++bits, t <<= 1)
-        ;
-    /*
-    ** If time_t is signed, then 0 is the median value,
-    ** if time_t is unsigned, then 1 << bits is median.
-    */
-    t = (t < 0) ? 0 : ((time_t) 1 << bits);
-
-    /* Some gmtime() implementations are broken and will return
-     * NULL for time_ts larger than 40 bits even on 64-bit platforms
-     * so we'll just cap it at 40 bits */
-    if(bits > 40) bits = 40;
-
-    for ( ; ; ) {
-        mytm = gmtime(&t);
-
-        if(!mytm) return WRONG;
-
-        dir = tmcomp(mytm, &yourtm);
-        if (dir != 0) {
-            if (bits-- < 0)
-                return WRONG;
-            if (bits < 0)
-                --t;
-            else if (dir > 0)
-                t -= (time_t) 1 << bits;
-            else  t += (time_t) 1 << bits;
-            continue;
-        }
-        break;
-    }
-    t += saved_seconds;
-    return t;
-}
-#define GMTIME_INVERSE(time_struct) mkgmtime(time_struct)
+#define INT2STRING(buffer, i)                                           \
+    *(buffer) = malloc(_scprintf("%d", (i)) + 1),                       \
+        (!(buffer) ?                                                    \
+         -1 :                                                           \
+         _snprintf(*(buffer),                                           \
+                     _scprintf("%d", (i)) + 1,                          \
+                     "%d",                                              \
+                     (i)))
+#define STRCAT(dest, n, src) strcat((dest), (src))
 #endif
-
-#define INT2STRING(buffer, i)                   \
-    {                                           \
-        int vslength = _scprintf("%d", i) + 1;  \
-        *buffer = malloc(vslength);             \
-        _snprintf(*buffer, vslength, "%d", i);  \
-    }
 #else
-#define GMTIME_INVERSE(time_struct) timegm(time_struct)
-#define INT2STRING(buffer, i) asprintf(buffer, "%d", i);
+#define INT2STRING(buffer, i) asprintf((buffer), "%d", (i))
+#define STRCAT(dest, n, src) strcat((dest), (src))
 #endif
 
 /* A buffer representing some data being encoded to BSON. */
@@ -339,17 +198,6 @@ static int write_string(bson_buffer* buffer, PyObject* py_string) {
     return 1;
 }
 
-/* returns 0 on invalid ascii */
-static int validate_ascii(const char* data, int length) {
-    int i;
-    for (i = 0; i < length; i++) {
-        if (data[i] & 0x80) {
-            return 0;
-        }
-    }
-    return 1;
-}
-
 /* TODO our platform better be little-endian w/ 4-byte ints! */
 /* Write a single value to the buffer (also write it's type_byte, for which
  * space has already been reserved.
@@ -417,8 +265,7 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
             if (type_byte == -1) {
                 return 0;
             }
-            INT2STRING(&name, i);
-            if (!name) {
+            if (INT2STRING(&name, i) < 0 || !name) {
                 PyErr_NoMemory();
                 return 0;
             }
@@ -548,8 +395,10 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         int result;
 
         *(buffer->buffer + type_byte) = 0x02;
-        if (!validate_ascii(PyString_AsString(value), PyString_Size(value))) {
-            PyErr_SetString(InvalidStringData, "strings in documents must be ASCII only");
+        if (!is_legal_utf8_string((const unsigned char*)PyString_AsString(value),
+                                  PyString_Size(value))) {
+            PyErr_SetString(InvalidStringData,
+                            "strings in documents must be valid UTF-8");
             return 0;
         }
         result = write_string(buffer, value);
@@ -568,18 +417,20 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         return result;
     } else if (PyDateTime_CheckExact(value)) {
         time_t rawtime;
-        struct tm* timeinfo;
+        struct tm timeinfo;
         long long time_since_epoch;
 
         time(&rawtime);
-        timeinfo = localtime(&rawtime);
-        timeinfo->tm_year = PyDateTime_GET_YEAR(value) - 1900;
-        timeinfo->tm_mon = PyDateTime_GET_MONTH(value) - 1;
-        timeinfo->tm_mday = PyDateTime_GET_DAY(value);
-        timeinfo->tm_hour = PyDateTime_DATE_GET_HOUR(value);
-        timeinfo->tm_min = PyDateTime_DATE_GET_MINUTE(value);
-        timeinfo->tm_sec = PyDateTime_DATE_GET_SECOND(value);
-        time_since_epoch = GMTIME_INVERSE(timeinfo);
+        if (LOCALTIME(&timeinfo, &rawtime)) {
+            return 0;
+        }
+        timeinfo.tm_year = PyDateTime_GET_YEAR(value) - 1900;
+        timeinfo.tm_mon = PyDateTime_GET_MONTH(value) - 1;
+        timeinfo.tm_mday = PyDateTime_GET_DAY(value);
+        timeinfo.tm_hour = PyDateTime_DATE_GET_HOUR(value);
+        timeinfo.tm_min = PyDateTime_DATE_GET_MINUTE(value);
+        timeinfo.tm_sec = PyDateTime_DATE_GET_SECOND(value);
+        time_since_epoch = GMTIME_INVERSE(&timeinfo);
         time_since_epoch = time_since_epoch * 1000;
         time_since_epoch += PyDateTime_DATE_GET_MICROSECOND(value) / 1000;
 
@@ -623,7 +474,7 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         PyObject* py_pattern;
         PyObject* encoded_pattern;
         long int_flags;
-        char flags[7];
+        char flags[FLAGS_SIZE];
         int pattern_length,
             flags_length;
 
@@ -661,22 +512,22 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         flags[0] = 0;
         /* TODO don't hardcode these */
         if (int_flags & 2) {
-            strcat(flags, "i");
+            STRCAT(flags, FLAGS_SIZE, "i");
         }
         if (int_flags & 4) {
-            strcat(flags, "l");
+            STRCAT(flags, FLAGS_SIZE, "l");
         }
         if (int_flags & 8) {
-            strcat(flags, "m");
+            STRCAT(flags, FLAGS_SIZE, "m");
         }
         if (int_flags & 16) {
-            strcat(flags, "s");
+            STRCAT(flags, FLAGS_SIZE, "s");
         }
         if (int_flags & 32) {
-            strcat(flags, "u");
+            STRCAT(flags, FLAGS_SIZE, "u");
         }
         if (int_flags & 64) {
-            strcat(flags, "x");
+            STRCAT(flags, FLAGS_SIZE, "x");
         }
         flags_length = strlen(flags) + 1;
         if (!buffer_write_bytes(buffer, flags, flags_length)) {
@@ -746,7 +597,7 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         PyObject* errmsg = PyString_FromString("Cannot encode object: ");
         PyObject* repr = PyObject_Repr(value);
         PyString_ConcatAndDel(&errmsg, repr);
-        PyErr_SetString(CBSONError, PyString_AsString(errmsg));
+        PyErr_SetString(InvalidDocument, PyString_AsString(errmsg));
         Py_DECREF(errmsg);
         return 0;
     }
@@ -812,6 +663,13 @@ static int decode_and_write_pair(bson_buffer* buffer, PyObject* key,
     } else if (PyString_Check(key)) {
         encoded = key;
         Py_INCREF(encoded);
+
+        if (!is_legal_utf8_string((const unsigned char*)PyString_AsString(encoded),
+                                  PyString_Size(encoded))) {
+            PyErr_SetString(InvalidStringData,
+                            "strings in documents must be valid UTF-8");
+            return 0;
+        }
     } else {
         PyObject* errmsg = PyString_FromString("documents must have only string keys, key was ");
         PyObject* repr = PyObject_Repr(key);
@@ -915,6 +773,11 @@ static int write_dict(bson_buffer* buffer, PyObject* dict, unsigned char check_k
         return 0;
     }
     length = buffer->position - start_position;
+    if (length > 4 * 1024 * 1024) {
+        PyErr_SetString(InvalidDocument, "document too large - "
+                        "BSON documents are limited to 4 MB");
+        return 0;
+    }
     memcpy(buffer->buffer + length_location, &length, 4);
     return 1;
 }
@@ -1200,19 +1063,21 @@ static PyObject* get_value(const char* buffer, int* position, int type) {
             long long millis;
             int microseconds;
             time_t seconds;
-            struct tm* timeinfo;
+            struct tm timeinfo;
 
             memcpy(&millis, buffer + *position, 8);
             microseconds = (millis % 1000) * 1000;
             seconds = millis / 1000;
-            timeinfo = gmtime(&seconds);
+            if (GMTIME(&timeinfo, &seconds)) {
+                return NULL;
+            }
 
-            value = PyDateTime_FromDateAndTime(timeinfo->tm_year + 1900,
-                                               timeinfo->tm_mon + 1,
-                                               timeinfo->tm_mday,
-                                               timeinfo->tm_hour,
-                                               timeinfo->tm_min,
-                                               timeinfo->tm_sec,
+            value = PyDateTime_FromDateAndTime(timeinfo.tm_year + 1900,
+                                               timeinfo.tm_mon + 1,
+                                               timeinfo.tm_mday,
+                                               timeinfo.tm_hour,
+                                               timeinfo.tm_min,
+                                               timeinfo.tm_sec,
                                                microseconds);
             *position += 8;
             break;
@@ -1339,7 +1204,7 @@ static PyObject* get_value(const char* buffer, int* position, int type) {
             break;
         }
     default:
-        PyErr_SetString(CBSONError, "no c decoder for this type yet");
+        PyErr_SetString(InvalidDocument, "no c decoder for this type yet");
         return NULL;
     }
     return value;
@@ -1467,7 +1332,6 @@ PyMODINIT_FUNC init_cbson(void) {
     if (!module) {
         return;
     }
-    CBSONError = PyObject_GetAttrString(module, "InvalidDocument");
     InvalidName = PyObject_GetAttrString(module, "InvalidName");
     InvalidDocument = PyObject_GetAttrString(module, "InvalidDocument");
     InvalidStringData = PyObject_GetAttrString(module, "InvalidStringData");
