@@ -170,6 +170,9 @@ class Collection(object):
           - `safe` (optional): check that the insert succeeded?
           - `check_keys` (optional): check if keys start with '$' or
             contain '.', raising `pymongo.errors.InvalidName` in either case
+
+        .. versionchanged:: 1.1
+           Bulk insert works with any iterable
         """
         docs = doc_or_docs
         if isinstance(docs, types.DictType):
@@ -196,7 +199,9 @@ class Collection(object):
 
         There are many useful `update modifiers`_ which can be used when
         performing updates. For example, here we use the ``"$set"`` modifier to
-        modify some fields in a matching document::
+        modify some fields in a matching document:
+
+        .. doctest::
 
           >>> db.test.insert({"x": "y", "a": "b"})
           ObjectId('...')
@@ -272,6 +277,9 @@ class Collection(object):
            The `spec_or_object_id` parameter is now optional. If it is
            not specified *all* documents in the collection will be
            removed.
+
+        .. versionadded:: 1.1
+           The `safe` parameter.
         """
         spec = spec_or_object_id
         if spec is None:
@@ -379,6 +387,9 @@ class Collection(object):
             the cursor will continue from the last document received. For
             details, see the `tailable cursor documentation
             <http://www.mongodb.org/display/DOCS/Tailable+Cursors>`_.
+
+        .. versionadded:: 1.1
+           The `tailable` parameter.
         """
         if spec is None:
             spec = SON()
