@@ -21,7 +21,9 @@ to a replica pair use :meth:`~Connection.paired`.
    master-slave clusters.
 
 To get a :class:`~pymongo.database.Database` instance from a
-:class:`Connection` use either dictionary-style or attribute-style access::
+:class:`Connection` use either dictionary-style or attribute-style access:
+
+.. doctest::
 
   >>> from pymongo import Connection
   >>> c = Connection()
@@ -104,9 +106,12 @@ class Connection(object): # TODO support auth for pooling
             perform queries on a slave instance
           - `timeout` (optional): max time to wait when attempting to acquire a
             connection from the connection pool before raising an exception -
-            can be set to -1 to wait indefinitely
+            can be set to ``-1`` to wait indefinitely
           - `network_timeout` (optional): timeout (in seconds) to use for socket
             operations - default is no timeout
+
+        .. versionadded:: 1.1
+           The `network_timeout` parameter.
         """
         if host is None:
             host = self.HOST
